@@ -1,6 +1,6 @@
 <?php declare(strict_types = 0);
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -912,35 +912,34 @@ class CSvgGraph extends CSvg {
 					 */
 					$path_point = [];
 					foreach ($point as $type => $value) {
-						$in_range = ($max_value >= $value && $min_value <= $value);
-						if ($in_range || $metric['options']['type'] != SVG_GRAPH_TYPE_POINTS) {
-							$x = $this->canvas_x + $this->canvas_width
-								- $this->canvas_width * ($this->time_till - $clock + $timeshift) / $time_range;
+						$x = $this->canvas_x + $this->canvas_width
+							- $this->canvas_width * ($this->time_till - $clock + $timeshift) / $time_range;
 
-							if ($max_value - $min_value == INF) {
-								$y = $this->canvas_y + CMathHelper::safeMul([$this->canvas_height,
-									$max_value / 10 - $value / 10, 1 / ($max_value / 10 - $min_value / 10)
-								]);
-							}
-							else {
-								$y = $this->canvas_y + CMathHelper::safeMul([$this->canvas_height,
-									$max_value - $value, 1 / ($max_value - $min_value)
-								]);
-							}
-
-							if (!$in_range) {
-								$y = ($value > $max_value) ? max($y_min, $y) : min($y_max, $y);
-							}
-
-							$path_point[$type] = [
-								(int) ceil($x),
-								(int) ceil($y),
-								convertUnits([
-									'value' => $value,
-									'units' => $metric['units']
-								])
-							];
+						if ($max_value - $min_value == INF) {
+							$y = $this->canvas_y + CMathHelper::safeMul([$this->canvas_height,
+								$max_value / 10 - $value / 10, 1 / ($max_value / 10 - $min_value / 10)
+							]);
 						}
+						else {
+							$y = $this->canvas_y + CMathHelper::safeMul([$this->canvas_height,
+								$max_value - $value, 1 / ($max_value - $min_value)
+							]);
+						}
+
+						if ($value < $min_value || $value > $max_value) {
+							$y = $metric['options']['type'] == SVG_GRAPH_TYPE_POINTS
+								? CSvgGraphMetricsPoint::Y_OUT_OF_RANGE
+								: ($value > $max_value ? max($y_min, $y) : min($y_max, $y));
+						}
+
+						$path_point[$type] = [
+							(int) ceil($x),
+							(int) ceil($y),
+							convertUnits([
+								'value' => $value,
+								'units' => $metric['units']
+							])
+						];
 					}
 
 					$paths[$part_index][] = $path_point;
