@@ -101,27 +101,32 @@ class CLineGraphDraw extends CGraphDraw {
 	 * Add single item object to graph. If invalid 'delay' interval passed method will interrupt current request with
 	 * error message.
 	 *
-	 * @param array  $graph_item                   Array of graph item properties.
-	 * @param string $graph_item['itemid']         Item id.
-	 * @param string $graph_item['type']           Item type.
-	 * @param string $graph_item['name']           Item host display name.
-	 * @param string $graph_item['hostname']       Item hostname.
-	 * @param string $graph_item['key_']           Item key_ field value.
-	 * @param string $graph_item['value_type']     Item value type.
-	 * @param string $graph_item['history']        Item history field value.
-	 * @param string $graph_item['trends']         Item trends field value.
-	 * @param string $graph_item['delay']          Item delay.
-	 * @param string $graph_item['master_itemid']  Master item id for item of type ITEM_TYPE_DEPENDENT.
-	 * @param string $graph_item['units']          Item units value.
-	 * @param string $graph_item['hostid']         Item host id.
-	 * @param string $graph_item['hostname']       Item host name.
-	 * @param string $graph_item['color']          Item presentation color.
-	 * @param int    $graph_item['drawtype']       Item presentation draw type, could be one of
-	 *                                             GRAPH_ITEM_DRAWTYPE_* constants.
-	 * @param int    $graph_item['yaxisside']      Item axis side, could be one of GRAPH_YAXIS_SIDE_* constants.
-	 * @param int    $graph_item['calc_fnc']       Item calculation function, could be one of CALC_FNC_* constants.
-	 * @param int    $graph_item['calc_type']      Item graph presentation calculation type, GRAPH_ITEM_SIMPLE or
-	 *                                             GRAPH_ITEM_SUM.
+	 * @param array  $graph_item                               Array of graph item properties.
+	 * @param string $graph_item['itemid']                     Item id.
+	 * @param string $graph_item['type']                       Item type.
+	 * @param string $graph_item['name']                       Item host display name.
+	 * @param string $graph_item['hostname']                   Item hostname.
+	 * @param string $graph_item['key_']                       Item key_ field value.
+	 * @param string $graph_item['value_type']                 Item value type.
+	 * @param string $graph_item['history']                    Item history field value.
+	 * @param string $graph_item['trends']                     Item trends field value.
+	 * @param string $graph_item['delay']                      Item delay.
+	 * @param string $graph_item['master_itemid']              Master item id for item of type ITEM_TYPE_DEPENDENT.
+	 * @param string $graph_item['units']                      Item units value.
+	 * @param string $graph_item['hostid']                     Item host id.
+	 * @param string $graph_item['hostname']                   Item host name.
+	 * @param string $graph_item['color']                      Item presentation color.
+	 * @param int    $graph_item['drawtype']                   Item presentation draw type, could be one of
+	 *                                                         GRAPH_ITEM_DRAWTYPE_* constants.
+	 * @param int    $graph_item['yaxisside']                  Item axis side, could be one of GRAPH_YAXIS_SIDE_*
+	 *                                                         constants.
+	 * @param int    $graph_item['calc_fnc']                   Item calculation function, could be one of CALC_FNC_*
+	 *                                                         constants.
+	 * @param int    $graph_item['calc_type']                  Item graph presentation calculation type,
+	 *                                                         GRAPH_ITEM_SIMPLE or GRAPH_ITEM_SUM.
+	 * @param array  $graph_item['preprocessing']              Item preprocessing steps.
+	 * @param string $graph_item['preprocessing'][]['type']    Item preprocessing step type.
+	 * @param string $graph_item['preprocessing'][]['params']  Item preprocessing step parameters.
 	 */
 	public function addItem(array $graph_item) {
 		if ($this->type == GRAPH_TYPE_STACKED) {
@@ -290,6 +295,10 @@ class CLineGraphDraw extends CGraphDraw {
 			$this->sizeX
 		);
 
+		$results = self::populateValuesBetweenHeartbeats($items, $this->from_time, $this->to_time, $this->sizeX,
+			$results
+		);
+
 		foreach ($items as $item) {
 			$data = [
 				'count' => [],
@@ -426,6 +435,77 @@ class CLineGraphDraw extends CGraphDraw {
 		}
 	}
 
+	private static function populateValuesBetweenHeartbeats(array $items, int $time_from, int $time_to, int $width,
+			array $results) {
+		foreach ($items as $item) {
+			if ($item['throttling_type'] != ZBX_PREPROC_THROTTLE_VALUE
+					&& $item['throttling_type'] != ZBX_PREPROC_THROTTLE_TIMED_VALUE) {
+				continue;
+			}
+
+			if ($item['delay'] === null || $item['delay'] == 0) {
+				continue;
+			}
+
+			$throttling_delay = null;
+
+			if ($item['throttling_type'] == ZBX_PREPROC_THROTTLE_TIMED_VALUE
+					&& ($throttling_delay = timeUnitToSeconds($item['throttling_delay'])) === null) {
+				continue;
+			}
+
+			$delay = $item['source'] === 'trends' ? max($item['delay'], ZBX_MAX_TREND_DIFF) : $item['delay'];
+			$period = $time_to - $time_from;
+
+			if (($delay_width = $delay * $width / $period) < 1) {
+				$delay = (int) ceil($delay / $delay_width);
+				$delay_width = $delay * $width / $period;
+			}
+
+			$prev_data = null;
+			$new_data = [];
+
+			foreach ($results[$item['itemid']]['data'] as $data) {
+				if ($prev_data != null && $data['clock'] - $prev_data['clock'] > $delay && ($throttling_delay === null
+						|| $data['clock'] - $prev_data['clock'] <= 2 * $throttling_delay)) {
+					$n = round(($data['clock'] - $prev_data['clock']) / $delay) - 1;
+					for ($i = 1; $i < $n; $i += 3) {
+						$new_data[] = [
+							'i' => round($prev_data['i'] + $i * $delay_width),
+							'clock' => $prev_data['clock'] + $i * $delay
+						] + $prev_data;
+					}
+					$new_data[] = [
+						'i' => round($prev_data['i'] + $n * $delay_width),
+						'clock' => $prev_data['clock'] + $n * $delay
+					] + $prev_data;
+				}
+
+				$new_data[] = $data;
+				$prev_data = $data;
+			}
+
+			if ($prev_data != null && $time_to - $prev_data['clock'] > $delay && ($throttling_delay === null
+					|| $time_to - $prev_data['clock'] <= 2 * $throttling_delay)) {
+				$n = round(($time_to - $prev_data['clock']) / $delay);
+				for ($i = 1; $i < $n; $i += 3) {
+					$new_data[] = [
+						'i' => round($prev_data['i'] + $i * $delay_width),
+						'clock' => $prev_data['clock'] + $i * $delay
+					] + $prev_data;
+				}
+				$new_data[] = [
+					'i' => round($prev_data['i'] + $n * $delay_width),
+					'clock' => $prev_data['clock'] + $n * $delay
+				] + $prev_data;
+			}
+
+			$results[$item['itemid']]['data'] = $new_data;
+		}
+
+		return $results;
+	}
+
 	protected function selectTriggers() {
 		$this->triggers = [];
 
@@ -2017,66 +2097,6 @@ class CLineGraphDraw extends CGraphDraw {
 		return $min_dimensions;
 	}
 
-	/**
-	 * Expands graph item objects data: macros in item name, time units, dependent item
-	 */
-	private function expandItems() {
-		$items_cache = zbx_toHash($this->items, 'itemid');
-		$items = $this->items;
-
-		do {
-			$master_itemids = [];
-
-			foreach ($items as $item) {
-				if ($item['type'] == ITEM_TYPE_DEPENDENT && !array_key_exists($item['master_itemid'], $items_cache)) {
-					$master_itemids[$item['master_itemid']] = true;
-				}
-				$items_cache[$item['itemid']] = $item;
-			}
-			$master_itemids = array_keys($master_itemids);
-
-			$items = API::Item()->get([
-				'output' => ['itemid', 'type', 'master_itemid', 'delay'],
-				'itemids' => $master_itemids,
-				'filter' => [
-					'flags' => [ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_PROTOTYPE, ZBX_FLAG_DISCOVERY_CREATED]
-				]
-			]);
-		} while ($items);
-
-		$update_interval_parser = new CUpdateIntervalParser();
-
-		foreach ($this->items as &$graph_item) {
-			if ($graph_item['type'] == ITEM_TYPE_DEPENDENT) {
-				$master_item = $graph_item;
-
-				while ($master_item && $master_item['type'] == ITEM_TYPE_DEPENDENT) {
-					$master_item = $items_cache[$master_item['master_itemid']];
-				}
-				$graph_item['type'] = $master_item['type'];
-				$graph_item['delay'] = $master_item['delay'];
-			}
-
-			$graph_item = CMacrosResolverHelper::resolveTimeUnitMacros([$graph_item], ['delay'])[0];
-
-			$update_interval_parser->parse($graph_item['delay']);
-			$graph_item['delay'] = getItemDelay($update_interval_parser->getDelay(),
-				$update_interval_parser->getIntervals(ITEM_DELAY_FLEXIBLE)
-			);
-
-			$graph_item['has_scheduling_intervals']
-				= (bool) $update_interval_parser->getIntervals(ITEM_DELAY_SCHEDULING);
-
-			if (strpos($graph_item['units'], ',') === false) {
-				$graph_item['units_long'] = '';
-			}
-			else {
-				list($graph_item['units'], $graph_item['units_long']) = explode(',', $graph_item['units'], 2);
-			}
-		}
-		unset($graph_item);
-	}
-
 	/**
 	 * Calculate graph dimensions and draw 1x1 pixel image placeholder.
 	 */
@@ -2104,7 +2124,7 @@ class CLineGraphDraw extends CGraphDraw {
 
 		$this->calculateTopPadding();
 
-		$this->expandItems();
+		CGraphHelper::calculateMetricsDelay($this->items);
 		$this->selectTriggers();
 		$this->calcDimensions();
 
@@ -2129,7 +2149,7 @@ class CLineGraphDraw extends CGraphDraw {
 
 		// Correct item 'delay' field value when graph data requested for trends.
 		foreach ($this->items as &$item) {
-			if ($item['source'] === 'trends' && (!$item['has_scheduling_intervals'] || $item['delay'] != 0)) {
+			if ($item['source'] === 'trends' && $item['delay'] !== null) {
 				$item['delay'] = max($item['delay'], ZBX_MAX_TREND_DIFF);
 			}
 		}
@@ -2160,10 +2180,7 @@ class CLineGraphDraw extends CGraphDraw {
 
 				$delay = $this->items[$item]['delay'];
 
-				if ($this->items[$item]['type'] == ITEM_TYPE_TRAPPER
-						|| ($this->items[$item]['type'] == ITEM_TYPE_ZABBIX_ACTIVE
-							&& preg_match('/^(event)?log(rt)?\[/', $this->items[$item]['key_']))
-						|| ($this->items[$item]['has_scheduling_intervals'] && $delay == 0)) {
+				if ($delay === null) {
 					$draw = true;
 				}
 				else {
