@@ -25,13 +25,17 @@
 		init({qr_code_url}) {
 			const qr_code_div = document.querySelector('.qr-code');
 			const size = qr_code_div.clientWidth;
-
-			new QRCode(qr_code_div, {
+			const qr = new QRCode(qr_code_div, {
 				text: qr_code_url,
 				width: size,
 				height: size,
 				correctLevel : QRCode.CorrectLevel.L
 			});
+			const module_width = Math.ceil(size / qr._oQRCode.moduleCount);
+			const qr_margin_width = module_width * 4;
+			const margin_color = qr._htOption.colorLight;
+
+			qr_code_div.style.border = `${qr_margin_width}px solid ${margin_color}`;
 		}
 	}
 </script>
